!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Information System (LIS) v7.1
!
! Copyright (c) 2015 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
module LIS_ObjFunc_pluginMod
!BOP
!
! !MODULE: LIS_ObjFunc_pluginMod
! 
! !DESCRIPTION: 
!   This module contains the definition of the functions that specify
!   different methods (metrics) to compute objective function for use
!   in optimization. 
! 
!   The user defined functions are incorporated into 
!   the appropriate registry to be later invoked through generic calls. 
!   
! !REVISION HISTORY: 
!  16 Jul 2009;   Sujay Kumar  Initial Specification
! 
!EOP  
  implicit none
  PRIVATE
!------------------------------------------------------------------------------
! !PUBLIC MEMBER FUNCTIONS:
!------------------------------------------------------------------------------
  PUBLIC :: LIS_ObjFunc_plugin    
contains
!BOP
! !ROUTINE: LIS_ObjFunc_plugin
!  \label{LIS_ObjFunc_plugin}
!
! !DESCRIPTION:
!
!  This is a plugin point for introducing different methods for objective
!  function computations (in parameter estimation) 
!
!
! !INTERFACE:
  subroutine LIS_ObjFunc_plugin
!EOP

    use LIS_pluginIndices

    use LSobjFunc_Mod, only : initializeLSObjFunc
#if 0
    use LMobjFunc_Mod, only : initializeLMObjFunc
#endif
    use LLobjFunc_Mod, only : initializeLLObjFunc
    use PobjFunc_Mod,  only : initializePObjFunc
    
    external updateLSestimate, computeLSestimate, resetLSestimate
#if 0
    external updateLMestimate, computeLMestimate, resetLMestimate
#endif
    external updateLLestimate, computeLLestimate, resetLLestimate
    external updatePestimate, computePestimate, resetPestimate

    
! These methods are specified for different type of objective 
! function evaluations

#if 0
    call registerinitobjfunctype(trim(LIS_LMestimateId)//char(0), initializeLMObjFunc)
    call registerupdateobjfunctype(trim(LIS_LMestimateId)//char(0), updateLMestimate)
    call registercomputeobjfunctype(trim(LIS_LMestimateId)//char(0), computeLMestimate)
    call registerresetobjfunctype(trim(LIS_LMestimateId)//char(0), resetLMestimate)
#endif

    call registerinitobjfunctype(trim(LIS_LSestimateId)//char(0), initializeLSObjFunc)
    call registerupdateobjfunctype(trim(LIS_LSestimateId)//char(0), updateLSestimate)
    call registercomputeobjfunctype(trim(LIS_LSestimateId)//char(0), computeLSestimate)
    call registerresetobjfunctype(trim(LIS_LSestimateId)//char(0), resetLSestimate)

    call registerinitobjfunctype(trim(LIS_LLestimateId)//char(0), initializeLLObjFunc)
    call registerupdateobjfunctype(trim(LIS_LLestimateId)//char(0), updateLLestimate)
    call registercomputeobjfunctype(trim(LIS_LLestimateId)//char(0), computeLLestimate)
    call registerresetobjfunctype(trim(LIS_LLestimateId)//char(0), resetLLestimate)

    call registerinitobjfunctype(trim(LIS_PestimateId)//char(0), initializePObjFunc)
    call registerupdateobjfunctype(trim(LIS_PestimateId)//char(0), updatePestimate)
    call registercomputeobjfunctype(trim(LIS_PestimateId)//char(0), computePestimate)
    call registerresetobjfunctype(trim(LIS_PestimateId)//char(0), resetPestimate)

  end subroutine LIS_ObjFunc_plugin
end module LIS_ObjFunc_pluginMod
